﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Animals
{
	// клас за създаване на котки, но с вграден контрол на раждаемостта.
	class Cat : Animal
	{
		// максимален брой на котките, които могат да бъдат създадени
		private const int maxCats = 2;
		// това е броят създадени котки. 
		public static int TotalCats
		{
			get
			{
				return totalCats;
			}
		}
		private static int totalCats = 0;

		// декларираме конструкторите на класа като private, за да не могат да бъдат
		// създавани директно обекти от клас Cat. За създаването им ще ползваме статичният
		// метод CreateCats и неговите overloads
		private Cat(string name)
			: base(name)
		{
		}
		private Cat(string name, bool isAsleep)
			: base(name, isAsleep)
		{
		}

		// тъй като конструкторът може да бъде извикан само в рамките на този клас,
		// създаването на обект от клас Cat може да стане само чрез следните два
		// метода, в които осъществяваме "контрола на раждаемостта". Това е Cat Factory.
		public static Cat CreateCat(string name)
		{
			return CreateCat(name, false);
		}
		public static Cat CreateCat(string name, bool isAsleep)
		{
			// проверяваме статичната променлива totalCats
			if (totalCats + 1 > maxCats)
				throw new AnimalException("Твърде много котки!");

			Cat cat = new Cat(name, isAsleep);

			totalCats++;

			return cat;
		}

		// имплементираме метода Sound дефиниран в ISoundable интерфейса
		// и в последствие деклариран като абстрактен в класа Animal
		public override void Sound()
		{
			Console.WriteLine("Мяу!");
		}
	}
}
